"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProcessInfo = void 0;
const child_process_js_1 = require("./child_process.js");
__exportStar(require("./child_process.js"), exports);
__exportStar(require("./process-info-node.js"), exports);
const path_1 = require("path");
const process_info_node_js_1 = require("./process-info-node.js");
const fs_1 = require("fs");
const promises_1 = require("fs/promises");
const json_file_js_1 = require("./json-file.js");
class ProcessInfo {
    dir;
    exclude;
    roots = new Set();
    uuids = new Map();
    files = new Map();
    pendingRoot = new Map();
    pendingParent = new Map();
    externalIDs = new Map();
    static async load({ dir = (0, path_1.resolve)(process.cwd(), '.tap/processinfo'), exclude = /(^|\\|\/)node_modules(\\|\/|$)/, }) {
        const pi = new ProcessInfo({ dir, exclude });
        await pi.load();
        return pi;
    }
    static loadSync({ dir = (0, path_1.resolve)(process.cwd(), '.tap/processinfo'), exclude = /(^|\\|\/)node_modules(\\|\/|$)/, }) {
        const pi = new ProcessInfo({ dir, exclude });
        pi.loadSync();
        return pi;
    }
    constructor({ dir = (0, path_1.resolve)(process.cwd(), '.tap/processinfo'), exclude = /(^|\\|\/)node_modules(\\|\/|$)/, } = {}) {
        this.dir = dir;
        this.exclude = exclude;
    }
    clear() {
        this.roots.clear();
        this.files.clear();
        this.uuids.clear();
        this.externalIDs.clear();
    }
    async save() {
        await (0, promises_1.mkdir)(this.dir, { recursive: true });
        const writes = [];
        for (const [uuid, info] of this.uuids.entries()) {
            const f = `${this.dir}/${uuid}.json`;
            writes.push((0, promises_1.writeFile)(f, JSON.stringify(info) + '\n', 'utf8'));
        }
        await Promise.all(writes);
    }
    async saveSync() {
        (0, fs_1.mkdirSync)(this.dir, { recursive: true });
        for (const [uuid, info] of this.uuids.entries()) {
            const f = `${this.dir}/${uuid}.json`;
            (0, fs_1.writeFileSync)(f, JSON.stringify(info) + '\n', 'utf8');
        }
    }
    async erase() {
        this.clear();
        await (0, promises_1.rm)(this.dir, { recursive: true });
    }
    eraseSync() {
        this.clear();
        (0, fs_1.rmSync)(this.dir, { recursive: true });
    }
    async load() {
        const promises = [];
        for (const entry of await (0, promises_1.readdir)(this.dir).catch(() => [])) {
            const uuid = (0, path_1.basename)(entry, '.json');
            if (this.uuids.has(uuid)) {
                continue;
            }
            const f = (0, path_1.resolve)(this.dir, entry);
            promises.push((0, json_file_js_1.safeJSON)(f).then(data => {
                if (!data.uuid || data.uuid !== uuid) {
                    return;
                }
                new process_info_node_js_1.ProcessInfoNode(data).link(this);
            }));
        }
        await Promise.all(promises);
        return this;
    }
    loadSync() {
        let entries;
        try {
            entries = (0, fs_1.readdirSync)(this.dir);
        }
        catch (_) {
            entries = [];
        }
        for (const entry of entries) {
            const uuid = (0, path_1.basename)(entry, '.json');
            if (this.uuids.has(uuid)) {
                continue;
            }
            const f = (0, path_1.resolve)(this.dir, entry);
            const data = (0, json_file_js_1.safeJSONSync)(f);
            if (!data.uuid || data.uuid !== uuid) {
                continue;
            }
            new process_info_node_js_1.ProcessInfoNode(data).link(this);
        }
        return this;
    }
    static get Node() {
        return process_info_node_js_1.ProcessInfoNode;
    }
    static get ProcessInfo() {
        return ProcessInfo;
    }
    static get spawn() {
        return child_process_js_1.spawn;
    }
    static get spawnSync() {
        return child_process_js_1.spawnSync;
    }
    static get exec() {
        return child_process_js_1.exec;
    }
    static get execSync() {
        return child_process_js_1.execSync;
    }
    static get execFile() {
        return child_process_js_1.execFile;
    }
    static get execFileSync() {
        return child_process_js_1.execFileSync;
    }
    static get fork() {
        return child_process_js_1.fork;
    }
}
exports.ProcessInfo = ProcessInfo;
//# sourceMappingURL=index.js.map