"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.nodeOptionsEnv = void 0;
const argv_to_node_options_js_1 = require("./argv-to-node-options.js");
const loader_paths_js_1 = require("./loader-paths.js");
const node_options_to_argv_js_1 = require("./node-options-to-argv.js");
// Check if our esm and/or cjs loaders are in the list,
// and only add them if not.
const hasCJSLoader = (args) => hasLoader(args, ['-r', '--require'], loader_paths_js_1.cjsMatch);
const hasESMLoader = (args) => hasLoader(args, ['--experimental-loader', '--loader'], loader_paths_js_1.esmMatch);
const addCJS = (args) => !hasCJSLoader(args) ? args.concat('--require=' + loader_paths_js_1.cjsLoader) : args;
const addESM = (args) => !hasESMLoader(args) ? args.concat('--loader=' + loader_paths_js_1.esmLoader) : args;
const cjsOnly = (args) => hasESMLoader(args) ? false : hasCJSLoader(args);
const hasLoader = (args, keys, test) => {
    for (let i = 0; i < args.length; i++) {
        const arg = args[i];
        // -r <value>
        // --loader <value>
        if (keys.includes(arg) && i < args.length - 1) {
            if (test(args[i + 1])) {
                return true;
            }
            else {
                continue;
            }
        }
        else if (arg.startsWith('--') && arg.includes('=')) {
            // --require=<value>
            // --loader=<value>
            const [k, ...rest] = arg.split('=');
            if (keys.includes(k) && test(rest.join('='))) {
                return true;
            }
            else {
                continue;
            }
        }
        else if (!arg.startsWith('--') &&
            arg.startsWith('-') &&
            arg.length > 2 &&
            keys.includes(arg.substring(0, 2))) {
            // -r<value>
            if (test(arg.substring(2))) {
                return true;
            }
            else {
                continue;
            }
        }
    }
    return false;
};
const nodeOptionsEnv = (env, args) => {
    const no = (0, node_options_to_argv_js_1.nodeOptionsToArgv)(env.NODE_OPTIONS);
    return (0, argv_to_node_options_js_1.argvToNodeOptions)(cjsOnly(args.concat(no)) ? addCJS(no) : addESM(no));
};
exports.nodeOptionsEnv = nodeOptionsEnv;
//# sourceMappingURL=node-options-env.js.map