"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.coverageOnProcessEnd = exports.register = exports.SESSION = void 0;
// start tracking coverage, unless disabled explicltly
// export so that we know to collect at the end of the process
const enabled = process.env._TAPJS_PROCESSINFO_COVERAGE_ !== '0';
const fs_1 = require("fs");
const inspector_1 = require("inspector");
const module_1 = require("module");
const url_1 = require("url");
const get_exclude_js_1 = require("./get-exclude.js");
exports.SESSION = undefined;
// NB: coverage exclusion is in addition to processinfo
// exclusion.  Only show coverage for a file we care
// about at least somewhat, but coverage is a subset.
const exclude = (0, get_exclude_js_1.getExclude)('_TAPJS_PROCESSINFO_COV_EXCLUDE_');
// This is a \n delimited list of files to show coverage for
// If not set, or empty, then coverage is included for all files
// that pass the exclusion RegExp filter. If included in this list,
// then coverage will be recorded, even if it matches exclude.
const cfEnv = process.env._TAPJS_PROCESSINFO_COV_FILES_ || '';
const coveredFiles = cfEnv.trim().split('\n').filter(f => !!f);
const fileCovered = (f, s, files = []) => {
    const testFiles = [f];
    if (s) {
        for (const src of s.sources || []) {
            testFiles.push((0, url_1.fileURLToPath)(src));
        }
    }
    if (!testFiles.some(f => files.includes(f)))
        return false;
    // if at least one of them are explicitly covered, then include it,
    // otherwise omit if we explicitly listed
    if (coveredFiles.length) {
        return testFiles.some(f => coveredFiles.includes(f));
    }
    for (const f of testFiles) {
        if (!exclude.test(f))
            return true;
    }
    return false;
};
// C8 can't see that this function runs, best theory is that it
// collides with what it's doing with the coverage it's collecting
// This ignore can possibly be removed once this is being tested
// with a version of tap that uses this library, but it might just
// be an unresolveable bootstrap problem.
// The test does verify that it ran, because otherwise, there would
// be no coverage, and it verifies that it gets the expected coverage.
/* c8 ignore start */
const register = () => {
    if (!enabled)
        return;
    process.env._TAPJS_PROCESSINFO_COVERAGE_ = '1';
    exports.SESSION = new inspector_1.Session();
    exports.SESSION.connect();
    exports.SESSION.post('Profiler.enable');
    exports.SESSION.post('Runtime.enable');
    exports.SESSION.post('Profiler.startPreciseCoverage', {
        callCount: true,
        detailed: true,
    });
};
exports.register = register;
/* c8 ignore stop */
// only read the file again if we don't already have the content
// in the source map itself.
const lineLengths = (f, content) => !content
    ? lineLengths(f, (0, fs_1.readFileSync)(f, 'utf8'))
    : content.split(/\n|\u2028|\u2029/).map(l => l.length);
const coverageOnProcessEnd = (cwd, processInfo) => {
    // Similar to the coverage tracking bootstrap problem above, c8
    // doesn't see that this function runs, even though it DOES see
    // that the function defined below runs, which is weird.
    /* c8 ignore start */
    if (!exports.SESSION)
        return;
    const session = exports.SESSION;
    const f = `${cwd}/.tap/coverage/${processInfo.uuid}.json`;
    (0, fs_1.mkdirSync)(`${cwd}/.tap/coverage`, { recursive: true });
    session.post('Profiler.takePreciseCoverage', (er, cov) => {
        session.post('Profiler.stopPreciseCoverage');
        /* c8 ignore stop */
        // something very strange and bad happened
        /* c8 ignore start */
        if (er) {
            throw er;
        }
        /* c8 ignore stop */
        // Create a source-map-cache that c8 uses in report generation
        const sourceMapCache = {};
        Object.assign(cov, {
            'source-map-cache': sourceMapCache,
        });
        cov.result = cov.result.filter(obj => {
            if (!/^file:/.test(obj.url)) {
                return false;
            }
            const f = (0, url_1.fileURLToPath)(obj.url);
            // see if it has a source map
            const s = (0, module_1.findSourceMap)(f);
            if (!fileCovered(f, s?.payload, processInfo.files)) {
                return false;
            }
            if (s) {
                const { payload } = s;
                sourceMapCache[obj.url] = Object.assign(Object.create(null), {
                    lineLengths: lineLengths(f, payload.sourcesContent?.join('')),
                    data: payload,
                });
            }
            return true;
        });
        (0, fs_1.writeFileSync)(f, JSON.stringify(cov, null, 2) + '\n', 'utf8');
        /* c8 ignore start */
    });
};
exports.coverageOnProcessEnd = coverageOnProcessEnd;
/* c8 ignore stop */
//# sourceMappingURL=register-coverage.js.map